from sys import stdin

from promplate.prompt.chat import Message, assistant, system, user
from typer import Argument, Typer

from .impl import default_model, get_client
from .utils import get_user_message

app = Typer()


@app.command()
def ask(message: str = Argument(""), model: str = default_model):
    if not message:
        message = get_user_message()

    messages: list[Message] = [user > message]
    if not stdin.isatty():
        messages.insert(0, system > stdin.read())

    while True:
        out = ""
        for i in get_client().generate(messages, model=model):
            out += i
            print(i, end="", flush=True)
        print()

        messages.append(assistant > out)

        messages.append(user > get_user_message())



from contextlib import redirect_stderr
from functools import cache
from io import StringIO
from sys import stdin

from typer import Exit


@cache
def _get_session():
    from prompt_toolkit import PromptSession
    from prompt_toolkit.input import create_input

    with redirect_stderr(io := StringIO()):
        session = PromptSession()

    if "Input is not a terminal (fd=0)" in io.getvalue():  # Warning: Input is not a terminal (fd=0).
        return PromptSession(input=create_input(open("/dev/tty")))  # noqa: PTH123, SIM115

    return session


def prompt(*args, **kwargs):
    return _get_session().prompt(*args, **kwargs)


def _input(message: str):
    if stdin.isatty():
        return input(message)

    return prompt(message)


def get_user_message():
    print()

    message = _input(" > ")
    if not message:
        print()
        raise Exit

    print()

    return message
